﻿using Ecng.Common;
using Ecng.Configuration;
using StockSharp.Algo.Storages;
using StockSharp.Algo.Storages.Csv;
using StockSharp.BusinessEntities;
using StockSharp.InteractiveBrokers;
using StockSharp.Logging;
using StockSharp.Messages;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace CreateStorage
{
    class Program
    {
        private static LogManager _logManager = new LogManager();
        private static InteractiveBrokersTrader _connector;
        private static IEntityRegistry _entityRegistry;
        private static IStorageRegistry _storageRegistry;
        private static Portfolio _portfolio;
        private static Order _marketOrder;

        static void Main(string[] args)
        {
            var fileLogListener = new FileLogListener("connector_log.txt");
            fileLogListener.MaxLength = 1024000;
            fileLogListener.MaxCount = 100;
            fileLogListener.LogDirectory = @".\Logs";

            _connector = new InteractiveBrokersTrader();
            _connector.Address = "127.0.0.1:7497".To<EndPoint>();
            _connector.LogLevel = LogLevels.Debug;
            _logManager.Sources.Add(_connector);
            _logManager.Listeners.Add(fileLogListener);


            _connector.Connected += _connector_Connected;
            _connector.Disconnected += _connector_Disconnected;

            _connector.ConnectionError += _connector_ConnectionError;
            _connector.Error += _connector_Error;
            _connector.MarketDataSubscriptionFailed += _connector_MarketDataSubscriptionFailed;

            _connector.NewPortfolio += _connector_NewPortfolio;
            _connector.PortfolioChanged += _connector_PortfolioChanged;

            _connector.SecurityChanged += _connector_SecurityChanged;

            _connector.NewMyTrade += _connector_NewMyTrade;

            var dataPath = @"Data";
            _entityRegistry = new CsvEntityRegistry(dataPath);

            _storageRegistry = new StorageRegistry
            {
                DefaultDrive = new LocalMarketDataDrive(dataPath)
            };

            var adapter = _connector.Adapter.InnerAdapters.OfType<InteractiveBrokersMessageAdapter>().First();

            _connector.InitializeStorage(_entityRegistry, _storageRegistry);

            _connector.Adapter.InnerAdapters.Add(adapter);

            _entityRegistry.Init();

            _connector.StorageAdapter.DaysLoad = TimeSpan.FromDays(3);

            _connector.StorageAdapter.Format = StorageFormats.Csv;

            _connector.StorageAdapter.Load();

            _connector.Connect();

            while (Console.Read() != 'q')
            {
                Thread.Sleep(1000);
            }

            while (_marketOrder == null)
            {
                RegisterOrder();
                Thread.Sleep(5000);
            }
            
            while (Console.Read() != 'q')
            {
                Thread.Sleep(1000);
            }
        }

        private static void _connector_NewMyTrade(StockSharp.BusinessEntities.MyTrade obj)
        {
            Console.WriteLine("_connector_NewMyTrade:" + obj);
        }

        private static void _connector_SecurityChanged(StockSharp.BusinessEntities.Security obj)
        {
            Console.WriteLine("_connector_SecurityChanged:" + obj);
        }

        private static void _connector_PortfolioChanged(StockSharp.BusinessEntities.Portfolio obj)
        {
            Console.WriteLine("_connector_PortfolioChanged:" + obj);
        }

        private static void _connector_NewPortfolio(StockSharp.BusinessEntities.Portfolio obj)
        {
            Console.WriteLine("_connector_NewPortfolio:" + obj);
            _portfolio = obj;
        }

        private static void _connector_MarketDataSubscriptionFailed(StockSharp.BusinessEntities.Security arg1, StockSharp.Messages.MarketDataMessage arg2, Exception arg3)
        {
            Console.WriteLine("_connector_MarketDataSubscriptionFailed:" + arg1 + ", " + arg2 + ", " + arg3);
        }

        private static void _connector_Error(Exception obj)
        {
            Console.WriteLine("_connector_Error:" + obj);
        }

        private static void _connector_ConnectionError(Exception obj)
        {
            Console.WriteLine("_connector_ConnectionError:" + obj);
        }

        private static void _connector_Disconnected()
        {
            Console.WriteLine("_connector_Disconnected");
        }

        private static void _connector_Connected()
        {
            Console.WriteLine("_connector_Connected");
            if (_portfolio != null)
            {
                _connector.RegisterPortfolio(_portfolio);
            }
        }

        private static void RegisterOrder()
        {
            using (var waitHandle = new AutoResetEvent(false))
            {
                var criteria = new Security()
                {
                    Code = "SPX",
                    Board = new ExchangeBoard() { Code = "SMART" },
                    ExpiryDate = DateTimeOffset.Parse("2017-12-15T00:00:00+00:00"),
                    OptionType = OptionTypes.Call,
                    Strike = 2580M,
                    Type = SecurityTypes.Option
                };

                var onNewSecurity = new Action<Security>(s =>
                {
                    if ((!s.UnderlyingSecurityId.StartsWith(criteria.Code)) ||
                        (s.Type != criteria.Type) ||
                        (s.Board?.Code != criteria.Board.Code) ||
                        (s.ExpiryDate != criteria.ExpiryDate) ||
                        (s.OptionType != criteria.OptionType) ||
                        (s.Strike != criteria.Strike))
                        return;

                    waitHandle.Set();
                });

                _connector.NewSecurity += onNewSecurity;

                _connector.LookupSecurities(criteria);

                waitHandle.WaitOne(30000);

                _connector.NewSecurity -= onNewSecurity;

                var security = _connector.Securities.FirstOrDefault(s =>
                    (s.UnderlyingSecurityId.StartsWith(criteria.Code)) &&
                    (s.Board?.Code == criteria.Board.Code) &&
                    (s.Type == criteria.Type) &&
                    (s.ExpiryDate == criteria.ExpiryDate) &&
                    (s.OptionType == criteria.OptionType) &&
                    (s.Strike == criteria.Strike));

                if (security == null)
                {
                    return;
                }

                if (_connector.RegisteredSecurities.Count(s => s.Id == security.Id) == 0)
                {
                    var onSecurityChanged = new Action<Security>(s =>
                    {
                        if (s.Id != security.Id)
                            return;

                        if (s.BestBid != null && s.BestAsk != null)
                            waitHandle.Set();
                    });

                    _connector.SecurityChanged += onSecurityChanged;

                    _connector.RegisterSecurity(security);

                    waitHandle.WaitOne(30000);

                    _connector.SecurityChanged -= onSecurityChanged;
                }

                if ((security.BestBid == null) || (security.BestAsk == null))
                {
                    return;
                }

                _marketOrder = new Order()
                {
                    Security = security,
                    Portfolio = _portfolio,
                    Volume = 1,
                    Price = security.BestAsk.Price,
                    Direction = Sides.Buy,
                    UserOrderId = Guid.NewGuid().ToString(),
                    Type = OrderTypes.Limit
                };

                _connector.RegisterOrder(_marketOrder);
            }
        }
    }
}
